@@ -68,9 +68,9 @@ fn prefer_outer_delimiter(language: guess_language::Language) -> bool {
         // languages have syntax like `foo(bar)` or `foo[bar]` where
         // the inner delimiter is more relevant.
         Bash | C | CMake | CPlusPlus | CSharp | Css | Dart | Elixir | Elm | Elvish | Gleam | Go
-        | Hack | Haskell | Html | Java | JavaScript | Jsx | Julia | Kotlin | Lua | Nix | OCaml
-        | OCamlInterface | Perl | Php | Python | Ruby | Rust | Scala | Swift | Tsx | TypeScript
-        | Yaml | Zig => false,
+        | Hack | Haskell | Html | Java | JavaScript | Jsx | Julia | Kotlin | Lua | Make | Nix
+        | OCaml | OCamlInterface | Perl | Php | Python | Ruby | Rust | Scala | Swift | Tsx
+        | TypeScript | Yaml | Zig => false,
     }
 }
 
@@ -137,11 +137,11 @@ fn fix_nested_slider_prefer_outer<'a>(node: &'a Syntax<'a>, change_map: &mut Cha
                 }
             }
             ReplacedComment(_, _) => {}
-            Novel => {
-                for child in children {
-                    fix_nested_slider_prefer_outer(child, change_map);
-                }
-            }
+            Novel => {}
+        }
+
+        for child in children {
+            fix_nested_slider_prefer_outer(child, change_map);
         }
     }
 }
@@ -155,11 +155,7 @@ fn fix_nested_slider_prefer_inner<'a>(node: &'a Syntax<'a>, change_map: &mut Cha
             .get(node)
             .expect("Changes should be set before slider correction")
         {
-            Unchanged(_) => {
-                for child in children {
-                    fix_nested_slider_prefer_inner(child, change_map);
-                }
-            }
+            Unchanged(_) => {}
             ReplacedComment(_, _) => {}
             Novel => {
                 let mut found_unchanged = vec![];
@@ -170,6 +166,10 @@ fn fix_nested_slider_prefer_inner<'a>(node: &'a Syntax<'a>, change_map: &mut Cha
                 }
             }
         }
+
+        for child in children {
+            fix_nested_slider_prefer_inner(child, change_map);
+        }
     }
 }
 
@@ -396,6 +396,10 @@ fn novel_regions_before_unchanged<'a>(
         }
     }
 
+    if let Some(region) = region {
+        regions.push(region);
+    }
+
     regions
         .into_iter()
         .filter(|r| !r.is_empty())
