@@ -32,14 +32,17 @@ extern crate log;
 
 use crate::diff::{dijkstra, unchanged};
 use crate::display::hunks::{matched_pos_to_hunks, merge_adjacent};
+use crate::parse::guess_language::LANG_EXTENSIONS;
 use crate::parse::syntax;
 use diff::changes::ChangeMap;
 use diff::dijkstra::ExceededGraphLimit;
 use display::context::opposite_positions;
-use files::{is_probably_binary, read_files_or_die, read_or_die, relative_paths_in_either};
+use files::{
+    guess_content, read_files_or_die, read_or_die, relative_paths_in_either, ProbableFileKind,
+};
 use log::info;
 use mimalloc::MiMalloc;
-use parse::guess_language::guess;
+use parse::guess_language::{guess, language_name};
 
 /// The global allocator used by difftastic.
 ///
@@ -50,6 +53,7 @@ static GLOBAL: MiMalloc = MiMalloc;
 
 use diff::sliders::fix_all_sliders;
 use options::{DisplayMode, DisplayOptions, Mode};
+use owo_colors::OwoColorize;
 use rayon::prelude::*;
 use std::{env, path::Path};
 use summary::{DiffResult, FileContent};
@@ -124,6 +128,19 @@ fn main() {
                 }
             }
         }
+        Mode::ListLanguages => {
+            for (language, extensions) in LANG_EXTENSIONS {
+                print!("{}", language_name(*language).bold());
+
+                let mut extensions: Vec<&str> = (*extensions).into();
+                extensions.sort();
+
+                for extension in extensions {
+                    print!(" .{}", extension);
+                }
+                println!();
+            }
+        }
         Mode::Diff {
             graph_limit,
             byte_limit,
@@ -214,26 +231,25 @@ fn diff_file_content(
     byte_limit: usize,
     language_override: Option<parse::guess_language::Language>,
 ) -> DiffResult {
-    if is_probably_binary(lhs_bytes) || is_probably_binary(rhs_bytes) {
-        return DiffResult {
-            lhs_display_path: lhs_display_path.into(),
-            rhs_display_path: rhs_display_path.into(),
-            language: None,
-            lhs_src: FileContent::Binary(lhs_bytes.to_vec()),
-            rhs_src: FileContent::Binary(rhs_bytes.to_vec()),
-            lhs_positions: vec![],
-            rhs_positions: vec![],
-        };
-    }
+    let (mut lhs_src, mut rhs_src) = match (guess_content(lhs_bytes), guess_content(rhs_bytes)) {
+        (ProbableFileKind::Binary, _) | (_, ProbableFileKind::Binary) => {
+            return DiffResult {
+                lhs_display_path: lhs_display_path.into(),
+                rhs_display_path: rhs_display_path.into(),
+                language: None,
+                lhs_src: FileContent::Binary(lhs_bytes.to_vec()),
+                rhs_src: FileContent::Binary(rhs_bytes.to_vec()),
+                lhs_positions: vec![],
+                rhs_positions: vec![],
+            };
+        }
+        (ProbableFileKind::Text(lhs_src), ProbableFileKind::Text(rhs_src)) => (lhs_src, rhs_src),
+    };
 
     // TODO: don't replace tab characters inside string literals.
     let tab_as_spaces = " ".repeat(tab_width);
-    let mut lhs_src = String::from_utf8_lossy(lhs_bytes)
-        .to_string()
-        .replace('\t', &tab_as_spaces);
-    let mut rhs_src = String::from_utf8_lossy(rhs_bytes)
-        .to_string()
-        .replace('\t', &tab_as_spaces);
+    lhs_src = lhs_src.replace('\t', &tab_as_spaces);
+    rhs_src = rhs_src.replace('\t', &tab_as_spaces);
 
     // Ignore the trailing newline, if present.
     // TODO: highlight if this has changes (#144).
@@ -261,7 +277,7 @@ fn diff_file_content(
         return DiffResult {
             lhs_display_path: lhs_display_path.into(),
             rhs_display_path: rhs_display_path.into(),
-            language: lang_config.map(|l| l.name.into()),
+            language: language.map(|l| language_name(l).into()),
             lhs_src: FileContent::Text("".into()),
             rhs_src: FileContent::Text("".into()),
             lhs_positions: vec![],
@@ -311,10 +327,6 @@ fn diff_file_content(
                         break;
                     }
                 }
-
-                let language = language.unwrap();
-                fix_all_sliders(language, &lhs_section_nodes, &mut change_map);
-                fix_all_sliders(language, &rhs_section_nodes, &mut change_map);
             }
 
             if exceeded_graph_limit {
@@ -326,9 +338,19 @@ fn diff_file_content(
                     rhs_positions,
                 )
             } else {
+                // TODO: Make this .expect() unnecessary.
+                let language =
+                    language.expect("If we had a ts_lang, we must have guessed the language");
+                fix_all_sliders(language, &lhs, &mut change_map);
+                fix_all_sliders(language, &rhs, &mut change_map);
+
                 let lhs_positions = syntax::change_positions(&lhs, &change_map);
                 let rhs_positions = syntax::change_positions(&rhs, &change_map);
-                (Some(ts_lang.name.into()), lhs_positions, rhs_positions)
+                (
+                    Some(language_name(language).into()),
+                    lhs_positions,
+                    rhs_positions,
+                )
             }
         }
         None => {
