@@ -111,6 +111,7 @@ fn app() -> clap::Command<'static> {
                 .value_name("BACKGROUND")
                 .env("DFT_BACKGROUND")
                 .possible_values(["dark", "light"])
+                .default_value("dark")
                 .help("Set the background brightness. Difftastic will prefer brighter colours on dark backgrounds.")
         )
         .arg(
@@ -136,6 +137,10 @@ fn app() -> clap::Command<'static> {
                 .help("Override language detection. Inputs are assumed to have this file extension. When diffing directories, applies to all files.")
                 // TODO: support DFT_LANGUAGE for consistency
         )
+        .arg(
+            Arg::new("list-languages").long("list-languages")
+                .help("Print the all the languages supported by difftastic, along with their extensions.")
+        )
         .arg(
             Arg::new("byte-limit").long("byte-limit")
                 .takes_value(true)
@@ -191,6 +196,7 @@ pub enum Mode {
         /// The path that we should display for the RHS file.
         rhs_display_path: String,
     },
+    ListLanguages,
     DumpTreeSitter {
         path: String,
         language_override: Option<guess_language::Language>,
@@ -220,6 +226,10 @@ pub fn parse_args() -> Mode {
         None => None,
     };
 
+    if matches.is_present("list-languages") {
+        return Mode::ListLanguages;
+    }
+
     if let Some(path) = matches.value_of("dump-syntax") {
         return Mode::DumpSyntax {
             path: path.to_string(),
@@ -315,14 +325,13 @@ pub fn parse_args() -> Mode {
         ColorOutput::Auto
     };
 
-    let background_color = if let Some(background) = matches.value_of("background") {
-        if background == "light" {
-            BackgroundColor::Light
-        } else {
-            BackgroundColor::Dark
-        }
-    } else {
-        BackgroundColor::Dark
+    let background_color = match matches
+        .value_of("background")
+        .expect("Always present as we've given clap a default")
+    {
+        "dark" => BackgroundColor::Dark,
+        "light" => BackgroundColor::Light,
+        _ => unreachable!("clap has already validated the values"),
     };
 
     let syntax_highlight = matches.value_of("syntax-highlight") == Some("on");
@@ -377,7 +386,22 @@ pub fn parse_args() -> Mode {
 /// Choose the display width: try to autodetect, or fall back to a
 /// sensible default.
 fn detect_display_width() -> usize {
-    terminal_size::terminal_size().map_or(80, |(w, _)| w.0 as usize)
+    // terminal_size is actively maintained, but only considers
+    // stdout. This is a problem inside git, where stderr is a TTY
+    // with a size but stdout is piped to less.
+    //
+    // https://github.com/eminence/terminal-size/issues/23
+    if let Some(width) = terminal_size::terminal_size().map(|(w, _)| w.0 as usize) {
+        return width;
+    }
+
+    // term_size is no longer maintained, but it checks all of stdin,
+    // stdout and stderr, so gives better results in may cases.
+    if let Some(width) = term_size::dimensions().map(|(w, _)| w) {
+        return width;
+    }
+
+    80
 }
 
 pub fn should_use_color(color_output: ColorOutput) -> bool {
