@@ -1,4 +1,6 @@
-use crate::syntax::MatchedPos;
+//! Data types summarising the result of diffing content.
+
+use crate::parse::syntax::MatchedPos;
 
 #[derive(Debug, PartialEq, Eq)]
 pub enum FileContent {
@@ -8,7 +10,8 @@ pub enum FileContent {
 
 #[derive(Debug)]
 pub struct DiffResult {
-    pub path: String,
+    pub lhs_display_path: String,
+    pub rhs_display_path: String,
     pub language: Option<String>,
     pub lhs_src: FileContent,
     pub rhs_src: FileContent,
